/* 
 * File:   gff.h
 * Author: Aziz Mithani
 *
 * Created on April 7, 2016, 11:39 AM
 */

#ifndef GFF_H
#define	GFF_H

#include <map>
#include <set>
#include <string>
#include "gff_entry.h"

using namespace std;

class gff {
public:
    gff();
    gff(const gff& old_gff);
    gff(const string& gff_file);
    virtual ~gff();

    // type definition
    typedef vector<gff_entry*> SEQ_GFF_LIST;
    typedef map<string, SEQ_GFF_LIST* > GFF_LIST;

    // constants
    static const int OFFSET = 20;
    static const string FEATURE_EXON;
    static const string FEATURE_GENE;

    void read_gff_file(const string& gff_file);
    SEQ_GFF_LIST* get_gff_list(const string& sequence_name);
    void set_gff_list(const string& sequence_name, SEQ_GFF_LIST* seq_gff_list_ptr);
    static gff_entry* get_gff_entry(SEQ_GFF_LIST* seq_gff_list_ptr, int position);
    void get_sequence_names(set<string>& sequence_names);
    void create_index_array(map<string, int*>&  index_array);
    void delete_index_array(map<string, int*>&  index_array);
    void str_split(const string& str, vector<string>& tokens, const string& delimiters);
    
private:
    GFF_LIST gff_list;
};

#endif	/* GFF_H */

